# lib/Software/License/GPL_3/or_later.pm #

#   This file is part of Software-License-OrLaterPack.
#
#   Software-License-OrLaterPack is free software: you can redistribute it
#   and/or modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   Software-License-OrLaterPack is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
#   Public License for more details.
#
#   You should have received a copy of the GNU General Public License along
#   with Software-License-OrLaterPack.  If not, see
#   <http://www.gnu.org/licenses/>.

package Software::License::GPL_3::or_later;

# ABSTRACT: GPLv3+ license for Software::License

use strict;
use warnings;

# VERSION

use parent 'Software::License::GPL_3';
use Text::Wrap;

=method _name

Returns the name of license, with "the" article in front of name (in lower case). It does include
neither license version nor "or any later" clause.

=cut

sub _name { 'the GNU General Public License'; };

=method _version

Returns version of the license, just a number, no date, no plus sign at the end.

=cut

sub _version { '3'; };

=method name

This method, in contrast to L<_name>, meets <Software::License> specification: Definitive article
is capitalized ("The"), name includes version and "or any later" clause.

=cut

sub name {
    sprintf( "%s version %s, or any later", ucfirst( $_[ 0 ]->_name ), $_[ 0 ]->_version );
}; # sub name

=method version

Returns version with trailing plus sign.

=cut

sub version { $_[ 0 ]->_version . '+'; };

=method notice

This method overrides L<Software::License>'s C<notice>. Differences are:

=for :list
* The method has optional parameter — program name (default name is "this program"). It is used in
notice text.
* It returns copyright statement followed by standard GNU 3-paragraph license notice.
* Result is formatted with `Text::Wrap::fill`.

B<Note:> Program name is an experimental feature.

=cut

sub notice {
    my ( $self, $program ) = @_;
    #   Documentation on `fill` says it deletes all trailing whitespace, but it looks like it
    #   may leave one space. Let us make sure notice ends with one newline.
    $self->{ program } = defined( $program ) ? $program : 'this program';
    ( my $notice = Text::Wrap::fill( '', '', $self->SUPER::notice() ) ) =~ s{\s*\z}{\n};
    delete( $self->{ program } );
    return $notice;
}; # sub notice

1;

=pod

=head1 DESCRIPTION

C<Software::License::GPL_3::or_later> is a subclass of L<Software::License::GPL_3>. It overrides
few parent methods:

=for :list
= C<name>
appends ", or any later" clause to the parent license name;
= C<version>
appends "+" to the parent license version;
= C<notice>
returns copyright statement and standard GNU 3-paragraph license notice (see the section
L<COPYRIGHT AND LICENSE> below).

See documentation on L<Software::License> for a general description of class interface.

=cut

__DATA__

__[ BEGIN BLOCK ]__
# BEGIN_BLOCK: Notice

__NOTICE__
Copyright (C) {{ $self->year }} {{ $self->holder }}

{{ ucfirst( $self->{ program } ) }} is free software: you can redistribute it and/or modify it
under the terms of {{ $self->_name }} as published by the Free Software Foundation, either version
{{ $self->_version }} of the License, or (at your option) any later version.

{{ ucfirst( $self->{ program } ) }} is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See {{ $self->_name }} for more details.

You should have received a copy of {{ $self->_name }} along with {{ $self->{ program } }}. If not,
see <http://www.gnu.org/licenses/>.

__[ END BLOCK ]__
# END_BLOCK: Notice

# end of file #
